=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2010-12-18 04:43:40 +0000
+++ drizzled/drizzled.cc	2010-12-22 21:24:17 +0000
@@ -475,7 +475,7 @@
       break;
     }
     /* Close before unlock, avoiding crash. See LP bug#436685 */
-    list.front()->client->close();
+    list.front()->getClient()->close();
   }
 }
 
@@ -628,6 +628,14 @@
     session		 Thread handler
 */
 
+void drizzled::Session::unlink(session_id_t &session_id)
+{
+  Session::shared_ptr session= session::Cache::singleton().find(session_id);
+
+  if (session)
+    unlink(session);
+}
+
 void drizzled::Session::unlink(Session::shared_ptr &session)
 {
   connection_count.decrement();

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2010-12-18 18:24:57 +0000
+++ drizzled/enum.h	2010-12-22 21:24:17 +0000
@@ -311,6 +311,7 @@
 
 enum drizzled_error_code {
 
+  EE_OK=0,
   EE_ERROR_FIRST=1,
   EE_CANTCREATEFILE,
   EE_READ,

=== modified file 'drizzled/generator/session.h'
--- drizzled/generator/session.h	2010-12-02 05:20:13 +0000
+++ drizzled/generator/session.h	2010-12-22 17:46:46 +0000
@@ -23,6 +23,7 @@
 
 #include <boost/thread/mutex.hpp>
 #include "drizzled/session/cache.h"
+#include "drizzled/identifier/user.h"
 
 namespace drizzled {
 namespace generator {
@@ -31,10 +32,12 @@
 {
   session::Cache::list local_list;
   session::Cache::list::const_iterator iter;
+  identifier::User::const_reference user;
 
 public:
 
-  Session()
+  Session(identifier::User::const_reference arg) :
+    user(arg)
   {
     boost::mutex::scoped_lock scopedLock(session::Cache::singleton().mutex());
     local_list= session::Cache::singleton().getCache();
@@ -45,22 +48,22 @@
   {
   }
 
-  operator drizzled::Session::shared_ptr()
+  operator drizzled::Session::pointer()
   {
     while (iter != local_list.end())
     {
-      if (not (*iter)->isViewable())
+      drizzled::Session::pointer ret= (*iter).get();
+      iter++;
+
+      if (not ret->isViewable(user))
       {
-        iter++;
         continue;
       }
 
-      drizzled::Session::shared_ptr ret(*iter);
-      iter++;
       return ret;
     }
 
-    return drizzled::Session::shared_ptr();
+    return drizzled::Session::pointer();
   }
 };
 

=== modified file 'drizzled/identifier/user.h'
--- drizzled/identifier/user.h	2010-12-18 05:18:30 +0000
+++ drizzled/identifier/user.h	2010-12-22 16:40:49 +0000
@@ -39,6 +39,7 @@
 public:
   typedef boost::shared_ptr<User> shared_ptr;
   typedef boost::shared_ptr<const User> const_shared_ptr;
+  typedef const User& const_reference;
   static shared_ptr make_shared();
 
   enum PasswordType

=== modified file 'drizzled/plugin/authorization.cc'
--- drizzled/plugin/authorization.cc	2010-12-18 05:18:30 +0000
+++ drizzled/plugin/authorization.cc	2010-12-22 16:40:49 +0000
@@ -208,6 +208,13 @@
                                          const Session *session,
                                          bool send_error)
 {
+  return isAuthorized(*user_ctx, session, send_error);
+}
+
+bool plugin::Authorization::isAuthorized(drizzled::identifier::User::const_reference user_ctx,
+                                         const Session *session,
+                                         bool send_error)
+{
   drizzled::identifier::User::const_shared_ptr session_ctx= session->user();
 
   /* If we never loaded any authorization plugins, just return true */
@@ -218,7 +225,7 @@
   std::vector<plugin::Authorization *>::const_iterator iter=
     std::find_if(authorization_plugins.begin(),
                  authorization_plugins.end(),
-                 RestrictProcessFunctor(*user_ctx, *session_ctx));
+                 RestrictProcessFunctor(user_ctx, *session_ctx));
 
   /*
    * If iter is == end() here, that means that all of the plugins returned
@@ -234,6 +241,7 @@
     }
     return false;
   }
+
   return true;
 }
 

=== modified file 'drizzled/plugin/authorization.h'
--- drizzled/plugin/authorization.h	2010-12-18 05:18:30 +0000
+++ drizzled/plugin/authorization.h	2010-12-22 16:40:49 +0000
@@ -96,6 +96,11 @@
                            const Session *session,
                            bool send_error= true);
 
+  /** Server API method for checking process authorization */
+  static bool isAuthorized(drizzled::identifier::User::const_reference user_ctx,
+                           const Session *session,
+                           bool send_error= true);
+
   /**
    * Server API helper method for applying authorization tests
    * to a set of schema names (for use in the context of getSchemaNames

=== modified file 'drizzled/select_send.h'
--- drizzled/select_send.h	2010-12-18 04:43:40 +0000
+++ drizzled/select_send.h	2010-12-22 16:40:49 +0000
@@ -60,7 +60,7 @@
   bool send_fields(List<Item> &list)
   {
     bool res;
-    if (! (res= session->client->sendFields(&list)))
+    if (! (res= session->getClient()->sendFields(&list)))
       is_result_set_started= 1;
     return res;
   }
@@ -106,7 +106,7 @@
     Item *item;
     while ((item=li++))
     {
-      if (item->send(session->client, &buffer))
+      if (item->send(session->getClient(), &buffer))
       {
         my_message(ER_OUT_OF_RESOURCES, ER(ER_OUT_OF_RESOURCES), MYF(0));
         break;
@@ -119,7 +119,7 @@
     session->sent_row_count++;
     if (session->is_error())
       return true;
-    return session->client->flush();
+    return session->getClient()->flush();
   }
 };
 

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2010-12-18 18:24:57 +0000
+++ drizzled/session.cc	2010-12-22 21:24:17 +0000
@@ -379,7 +379,7 @@
 {
   this->checkSentry();
 
-  if (client->isConnected())
+  if (client and client->isConnected())
   {
     assert(security_ctx);
     if (global_system_variables.log_warnings)
@@ -390,12 +390,15 @@
                     security_ctx->username().c_str());
     }
 
-    disconnect(0, false);
+    disconnect();
   }
 
   /* Close connection */
-  client->close();
-  delete client;
+  if (client)
+  {
+    client->close();
+    delete client;
+  }
 
   if (cleanup_done == false)
     cleanup();
@@ -537,7 +540,7 @@
 {
   if (storeGlobals())
   {
-    disconnect(ER_OUT_OF_RESOURCES, true);
+    disconnect(ER_OUT_OF_RESOURCES);
     status_var.aborted_connects++;
     return true;
   }
@@ -548,7 +551,7 @@
 {
   if (initGlobals() || authenticate())
   {
-    disconnect(0, true);
+    disconnect();
     return;
   }
 
@@ -560,7 +563,7 @@
       break;
   }
 
-  disconnect(0, true);
+  disconnect();
 }
 
 bool Session::schedule(Session::shared_ptr &arg)
@@ -610,11 +613,9 @@
 /*
   Is this session viewable by the current user?
 */
-bool Session::isViewable() const
+bool Session::isViewable(identifier::User::const_reference user_arg) const
 {
-  return plugin::Authorization::isAuthorized(current_session->user(),
-                                             this,
-                                             false);
+  return plugin::Authorization::isAuthorized(user_arg, this, false);
 }
 
 
@@ -1651,7 +1652,7 @@
   }
 }
 
-void Session::disconnect(uint32_t errcode, bool should_lock)
+void Session::disconnect(enum drizzled_error_code errcode)
 {
   /* Allow any plugins to cleanup their session variables */
   plugin_sessionvar_cleanup(this);
@@ -1675,26 +1676,17 @@
     }
   }
 
-  /* Close out our connection to the client */
-  if (should_lock)
-    session::Cache::singleton().mutex().lock();
-
   setKilled(Session::KILL_CONNECTION);
 
   if (client->isConnected())
   {
-    if (errcode)
+    if (errcode != EE_OK)
     {
       /*my_error(errcode, ER(errcode));*/
       client->sendError(errcode, ER(errcode));
     }
     client->close();
   }
-
-  if (should_lock)
-  {
-    session::Cache::singleton().mutex().unlock();
-  }
 }
 
 void Session::reset_for_next_command()

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2010-12-18 18:26:11 +0000
+++ drizzled/session.h	2010-12-22 21:24:17 +0000
@@ -319,6 +319,9 @@
   typedef boost::unordered_map<std::string, util::Storable *, util::insensitive_hash, util::insensitive_equal_to> PropertyMap;
   typedef Session* Ptr;
   typedef boost::shared_ptr<Session> shared_ptr;
+  typedef const Session& const_reference;
+  typedef const Session* const_pointer;
+  typedef Session* pointer;
 
   /*
     MARK_COLUMNS_NONE:  Means mark_used_colums is not set and no indicator to
@@ -529,8 +532,11 @@
   static const char * const DEFAULT_WHERE;
 
   memory::Root warn_root; /**< Allocation area for warnings and errors */
+private:
   plugin::Client *client; /**< Pointer to client object */
 
+public:
+
   void setClient(plugin::Client *client_arg);
 
   plugin::Client *getClient()
@@ -601,7 +607,7 @@
   /**
    * Is this session viewable by the current user?
    */
-  bool isViewable() const;
+  bool isViewable(identifier::User::const_reference) const;
 
   /**
     Used in error messages to tell user in what part of MySQL we found an
@@ -1217,6 +1223,7 @@
    */
   static bool schedule(Session::shared_ptr&);
 
+  static void unlink(session_id_t &session_id);
   static void unlink(Session::shared_ptr&);
 
   /*
@@ -1430,11 +1437,10 @@
    * updates any status variables necessary.
    *
    * @param errcode	Error code to print to console
-   * @param should_lock 1 if we have have to lock LOCK_thread_count
    *
    * @note  For the connection that is doing shutdown, this is called twice
    */
-  void disconnect(uint32_t errcode, bool lock);
+  void disconnect(enum drizzled_error_code errcode= EE_OK);
 
   /**
    * Check if user exists and the password supplied is correct.

=== modified file 'drizzled/sql_error.cc'
--- drizzled/sql_error.cc	2010-11-08 20:00:45 +0000
+++ drizzled/sql_error.cc	2010-12-22 16:40:49 +0000
@@ -214,7 +214,7 @@
   field_list.push_back(new Item_return_int("Code",4, DRIZZLE_TYPE_LONG));
   field_list.push_back(new Item_empty_string("Message",DRIZZLE_ERRMSG_SIZE));
 
-  if (session->client->sendFields(&field_list))
+  if (session->getClient()->sendFields(&field_list))
     return true;
 
   DRIZZLE_ERROR *err;
@@ -234,11 +234,11 @@
       continue;
     if (idx > unit->select_limit_cnt)
       break;
-    session->client->store(warning_level_names[err->level].str,
-		           warning_level_names[err->level].length);
-    session->client->store((uint32_t) err->code);
-    session->client->store(err->msg, strlen(err->msg));
-    if (session->client->flush())
+    session->getClient()->store(warning_level_names[err->level].str,
+                                warning_level_names[err->level].length);
+    session->getClient()->store((uint32_t) err->code);
+    session->getClient()->store(err->msg, strlen(err->msg));
+    if (session->getClient()->flush())
       return(true);
   }
   session->my_eof();

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2010-12-11 17:34:45 +0000
+++ drizzled/sql_parse.cc	2010-12-22 16:40:49 +0000
@@ -282,16 +282,16 @@
   {
   case Diagnostics_area::DA_ERROR:
     /* The query failed, send error to log and abort bootstrap. */
-    session->client->sendError(session->main_da.sql_errno(),
+    session->getClient()->sendError(session->main_da.sql_errno(),
                                session->main_da.message());
     break;
 
   case Diagnostics_area::DA_EOF:
-    session->client->sendEOF();
+    session->getClient()->sendEOF();
     break;
 
   case Diagnostics_area::DA_OK:
-    session->client->sendOK();
+    session->getClient()->sendOK();
     break;
 
   case Diagnostics_area::DA_DISABLED:
@@ -299,7 +299,7 @@
 
   case Diagnostics_area::DA_EMPTY:
   default:
-    session->client->sendOK();
+    session->getClient()->sendOK();
     break;
   }
 

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2010-12-18 02:08:04 +0000
+++ drizzled/sql_table.cc	2010-12-23 07:35:41 +0000
@@ -1749,7 +1749,7 @@
   item->maybe_null = 1;
   field_list.push_back(item = new Item_empty_string("Msg_text", 255, cs));
   item->maybe_null = 1;
-  if (session->client->sendFields(&field_list))
+  if (session->getClient()->sendFields(&field_list))
     return true;
 
   for (table= tables; table; table= table->next_local)
@@ -1804,18 +1804,18 @@
     {
       char buff[FN_REFLEN + DRIZZLE_ERRMSG_SIZE];
       uint32_t length;
-      session->client->store(table_name);
-      session->client->store(operator_name);
-      session->client->store(STRING_WITH_LEN("error"));
+      session->getClient()->store(table_name);
+      session->getClient()->store(operator_name);
+      session->getClient()->store(STRING_WITH_LEN("error"));
       length= snprintf(buff, sizeof(buff), ER(ER_OPEN_AS_READONLY),
                        table_name);
-      session->client->store(buff, length);
+      session->getClient()->store(buff, length);
       transaction_services.autocommitOrRollback(session, false);
       session->endTransaction(COMMIT);
       session->close_thread_tables();
       lex->reset_query_tables_list(false);
       table->table=0;				// For query cache
-      if (session->client->flush())
+      if (session->getClient()->flush())
 	goto err;
       continue;
     }
@@ -1846,18 +1846,18 @@
       DRIZZLE_ERROR *err;
       while ((err= it++))
       {
-        session->client->store(table_name);
-        session->client->store(operator_name);
-        session->client->store(warning_level_names[err->level].str,
+        session->getClient()->store(table_name);
+        session->getClient()->store(operator_name);
+        session->getClient()->store(warning_level_names[err->level].str,
                                warning_level_names[err->level].length);
-        session->client->store(err->msg);
-        if (session->client->flush())
+        session->getClient()->store(err->msg);
+        if (session->getClient()->flush())
           goto err;
       }
       drizzle_reset_errors(session, true);
     }
-    session->client->store(table_name);
-    session->client->store(operator_name);
+    session->getClient()->store(table_name);
+    session->getClient()->store(operator_name);
 
     switch (result_code) {
     case HA_ADMIN_NOT_IMPLEMENTED:
@@ -1865,41 +1865,41 @@
 	char buf[ERRMSGSIZE+20];
 	uint32_t length=snprintf(buf, ERRMSGSIZE,
                              ER(ER_CHECK_NOT_IMPLEMENTED), operator_name);
-	session->client->store(STRING_WITH_LEN("note"));
-	session->client->store(buf, length);
+	session->getClient()->store(STRING_WITH_LEN("note"));
+	session->getClient()->store(buf, length);
       }
       break;
 
     case HA_ADMIN_OK:
-      session->client->store(STRING_WITH_LEN("status"));
-      session->client->store(STRING_WITH_LEN("OK"));
+      session->getClient()->store(STRING_WITH_LEN("status"));
+      session->getClient()->store(STRING_WITH_LEN("OK"));
       break;
 
     case HA_ADMIN_FAILED:
-      session->client->store(STRING_WITH_LEN("status"));
-      session->client->store(STRING_WITH_LEN("Operation failed"));
+      session->getClient()->store(STRING_WITH_LEN("status"));
+      session->getClient()->store(STRING_WITH_LEN("Operation failed"));
       break;
 
     case HA_ADMIN_REJECT:
-      session->client->store(STRING_WITH_LEN("status"));
-      session->client->store(STRING_WITH_LEN("Operation need committed state"));
+      session->getClient()->store(STRING_WITH_LEN("status"));
+      session->getClient()->store(STRING_WITH_LEN("Operation need committed state"));
       open_for_modify= false;
       break;
 
     case HA_ADMIN_ALREADY_DONE:
-      session->client->store(STRING_WITH_LEN("status"));
-      session->client->store(STRING_WITH_LEN("Table is already up to date"));
+      session->getClient()->store(STRING_WITH_LEN("status"));
+      session->getClient()->store(STRING_WITH_LEN("Table is already up to date"));
       break;
 
     case HA_ADMIN_CORRUPT:
-      session->client->store(STRING_WITH_LEN("error"));
-      session->client->store(STRING_WITH_LEN("Corrupt"));
+      session->getClient()->store(STRING_WITH_LEN("error"));
+      session->getClient()->store(STRING_WITH_LEN("Corrupt"));
       fatal_error=1;
       break;
 
     case HA_ADMIN_INVALID:
-      session->client->store(STRING_WITH_LEN("error"));
-      session->client->store(STRING_WITH_LEN("Invalid argument"));
+      session->getClient()->store(STRING_WITH_LEN("error"));
+      session->getClient()->store(STRING_WITH_LEN("Invalid argument"));
       break;
 
     default:				// Probably HA_ADMIN_INTERNAL_ERROR
@@ -1908,8 +1908,8 @@
         uint32_t length=snprintf(buf, ERRMSGSIZE,
                              _("Unknown - internal error %d during operation"),
                              result_code);
-        session->client->store(STRING_WITH_LEN("error"));
-        session->client->store(buf, length);
+        session->getClient()->store(STRING_WITH_LEN("error"));
+        session->getClient()->store(buf, length);
         fatal_error=1;
         break;
       }
@@ -1938,7 +1938,7 @@
     session->endTransaction(COMMIT);
     session->close_thread_tables();
     table->table=0;				// For query cache
-    if (session->client->flush())
+    if (session->getClient()->flush())
       goto err;
   }
 

=== modified file 'drizzled/statement/kill.cc'
--- drizzled/statement/kill.cc	2010-12-18 04:43:40 +0000
+++ drizzled/statement/kill.cc	2010-12-22 16:40:49 +0000
@@ -27,13 +27,13 @@
 namespace drizzled
 {
 
-static bool kill_one_thread(session_id_t id, bool only_kill_query)
+bool statement::Kill::kill(session_id_t id, bool only_kill_query)
 {
-  drizzled::Session::shared_ptr session= session::Cache::singleton().find(id);
+  drizzled::Session::shared_ptr session_param= session::Cache::singleton().find(id);
 
-  if (session and session->isViewable())
+  if (session_param and session_param->isViewable(*getSession()->user()))
   {
-    session->awake(only_kill_query ? Session::KILL_QUERY : Session::KILL_CONNECTION);
+    session_param->awake(only_kill_query ? Session::KILL_QUERY : Session::KILL_CONNECTION);
     return true;
   }
 
@@ -52,7 +52,7 @@
     return true;
   }
 
-  if (kill_one_thread(static_cast<session_id_t>(it->val_int()), session->lex->type & ONLY_KILL_QUERY))
+  if (kill(static_cast<session_id_t>(it->val_int()), session->lex->type & ONLY_KILL_QUERY))
   {
     session->my_ok();
   }

=== modified file 'drizzled/statement/kill.h'
--- drizzled/statement/kill.h	2010-12-18 04:43:40 +0000
+++ drizzled/statement/kill.h	2010-12-22 16:40:49 +0000
@@ -39,6 +39,9 @@
   {}
 
   bool execute();
+
+private:
+  bool kill(session_id_t id, bool only_kill_query);
 };
 
 } /* namespace statement */

=== modified file 'plugin/multi_thread/multi_thread.cc'
--- plugin/multi_thread/multi_thread.cc	2010-12-02 05:20:13 +0000
+++ plugin/multi_thread/multi_thread.cc	2010-12-22 21:24:17 +0000
@@ -55,14 +55,17 @@
 
   if (drizzled::internal::my_thread_init())
   {
-    session->disconnect(drizzled::ER_OUT_OF_RESOURCES, true);
+    session->disconnect(drizzled::ER_OUT_OF_RESOURCES);
     session->status_var.aborted_connects++;
-    killSessionNow(session);
-  }
-  boost::this_thread::at_thread_exit(&internal::my_thread_end);
-
-  session->thread_stack= (char*) &stack_dummy;
-  session->run();
+  }
+  else
+  {
+    boost::this_thread::at_thread_exit(&internal::my_thread_end);
+
+    session->thread_stack= (char*) &stack_dummy;
+    session->run();
+  }
+
   killSessionNow(session);
   // @todo remove hard spin by disconnection the session first from the
   // thread.

=== modified file 'plugin/session_dictionary/processlist.cc'
--- plugin/session_dictionary/processlist.cc	2010-12-20 21:16:48 +0000
+++ plugin/session_dictionary/processlist.cc	2010-12-23 07:35:41 +0000
@@ -50,7 +50,8 @@
 }
 
 ProcesslistTool::Generator::Generator(Field **arg) :
-  plugin::TableFunction::Generator(arg)
+  plugin::TableFunction::Generator(arg),
+  session_generator(*getSession().user())
 {
   now= time(NULL);
 }
@@ -61,7 +62,7 @@
 
 bool ProcesslistTool::Generator::populate()
 {
-  drizzled::Session::shared_ptr tmp;
+  drizzled::Session::pointer tmp;
 
   while ((tmp= session_generator))
   {

=== modified file 'plugin/utility_functions/kill.cc'
--- plugin/utility_functions/kill.cc	2010-12-02 05:20:13 +0000
+++ plugin/utility_functions/kill.cc	2010-12-22 16:40:49 +0000
@@ -48,7 +48,7 @@
     return 0;
   }
 
-  if (not die_session->isViewable())
+  if (not die_session->isViewable(*getSession().user()))
   {
     my_error(drizzled::ER_KILL_DENIED_ERROR, session_id_for_kill, MYF(0));
     return 0;

